#include "QUANTAinit.hxx"
#include "BlockingTCPServer.h"

const int CBTCPServer::OK = 1;
const int CBTCPServer::FAIL = 0;

CBTCPServer::CBTCPServer(int port, int numClients)
{
	QUANTAinit();
	m_NumOfClients = 0;
	m_BunchSocket = new CTCPClient*[numClients];
	m_Server = new QUANTAnet_blockingTcpServer_c; // obtain a tcp socket
	if (port != 0)
		m_Server->init(port); // bind the socket to the port & listen
}

CBTCPServer::~CBTCPServer()
{
	for (int i = 0; i < BUNCHOFCLIENTS; i++)
		delete m_BunchSocket[i];
	delete[] m_BunchSocket;

	if (m_Server)
		delete m_Server;
}

int CBTCPServer::SetAddress(int port)
{
	int status = m_Server->init(port);
	return status;
}

CTCPClient* CBTCPServer::Listen() // check for new connection from clients
{
	QUANTAnet_tcpClient_c* client = m_Server->checkForNewConnections(); // receive connections from clients
	if (client) {
		CTCPClient* tcpClient = new CTCPClient(client);
		m_BunchSocket[m_NumOfClients] = tcpClient;
		m_NumOfClients++;
		return tcpClient;
	}
	return NULL;
}

int CBTCPServer::Receive(char* recvmesg, int& dataSize) // receive data from all connected clients
{
	for (int i = 0; i < m_NumOfClients; i++) {
		int ds = dataSize;

		int status = m_BunchSocket[i]->Receive(recvmesg, ds, QUANTAnet_tcpClient_c::BLOCKING); // receive data
		if (status == QUANTAnet_tcpClient_c::OK) {
			dataSize = ds;
			return OK;
		}
	}
	return FAIL;
}

void CBTCPServer::ReplyToClients(char* sendmesg, int dataSize) // send data to all connected clients
{
	for (int i = 0; i < m_NumOfClients; i++) {
		int ds = dataSize;
		m_BunchSocket[i]->Send(sendmesg, ds); // send data
	}
}

void CBTCPServer::Close() // close down sockets
{
	for (int i = 0; i < m_NumOfClients; i++) {
		m_BunchSocket[i]->Close(); // close
	}
	m_Server->close();
}
